package	jp.co.sds_corp.hibiya.server;

import java.util.ResourceBundle;

import jp.co.sds_corp.hibiya.adapter.AdapterFactory;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.core.Interval;
import jp.co.sds_corp.util.daemon.core.TimerDaemon;
import jp.co.sds_corp.util.daemon.manager.DaemonManager;

/**
{@link IChecker}を起動するDaemonです。
*/
public class MonitorDaemon extends TimerDaemon implements ICheckerListener
{
	private static final long serialVersionUID = 1L;
	long		interval;
	ResourceBundle	res;

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		res = ResourceBundle.getBundle( "resource.server" );
		IChecker chk = (IChecker )getService();
		AdapterFactory.addChecker( chk );

		chk.setInterval( getInterval().getMilliSec() );
		chk.addCheckerListener( this );
		String text = res.getString( "checker.init" );
		String msg = chk.getType() + " [" + chk.getName() + "] " + text;
		CheckerChangeEvent ev = new CheckerChangeEvent(
			chk, IChecker.Status.INIT, IChecker.Status.INIT, null, msg );
		chk.fireCheckerChange( ev );
	}

	/**
	* 処理を開始します。
	*/
	public void start() {
		AdapterFactory.setStarted( true );
		super.start();
	}

	/**
	* {@link IChecker}の状態の変更を受信します。
	*/
	public void checkerChanged( CheckerChangeEvent ev ) {
		IChecker chk = (IChecker )getService();
		setName( chk.getName());
		if ( ev.getNewStatus() == ev.getOldStatus()) {
			return;
		}
		if ( ev.getNewStatus() == IChecker.Status.DEL ) {
			terminate();
			DaemonManager.getInstance().removeDaemon( getName() );
			return;
		}
		try {
			if ( getInterval().getMilliSec() != chk.getInterval()) {
				setInterval( new Interval( chk.getInterval() ));
				startTimer();
			}
		} catch ( Exception ex ) {
			CmnLog.error( this, "setInterval error", ex );
		}
	}

}
